<?php

declare(strict_types=1);

namespace Drupal\swiffy_slider;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Template\Attribute;
use Drupal\Core\Url;
use function array_filter;
use function is_null;

/**
 * Supplies render arrays for Swiffy options to be used in configuration forms.
 *
 * @internal
 */
class Configuration implements ConfigurationInterface {

  use StringTranslationTrait;

  /**
   * Configured default configuration URL.
   *
   * @var string
   */
  private string $defaultConfigurationURL;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Configuration object factory.
   */
  final public function __construct(ConfigFactoryInterface $configFactory) {
    $this->defaultConfigurationURL = $configFactory->get('swiffy_slider.settings')->get('configuration_url') ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function configurationUrlElement(string $defaultValue = NULL): array {
    return [
      '#type' => 'url',
      '#title' => $this->t('Configuration URL'),
      '#description' => $this->t('Copy and paste the URL from @target configuration page.', [
        '@target' => Link::fromTextAndUrl('Swiffy slider', Url::fromUri(self::CONFIGURATOR_URL))
          ->toString(),
      ]),
      '#maxlength' => 1000,
      '#default_value' => $defaultValue,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function toAttributes(string $configurationURL = NULL): Attribute {
    if (is_null($configurationURL) || $configurationURL === '') {
      $configurationURL = $this->defaultConfigurationURL;
    }
    $parsed_url = UrlHelper::parse($configurationURL);
    $attributes = new Attribute();
    if (isset($parsed_url['query'])) {
      $attributes['class'] = array_values(array_filter($parsed_url['query'], fn(string $key): bool => str_starts_with($key, 'slider-'), ARRAY_FILTER_USE_KEY));
      $attributes['class'][] = 'swiffy-slider';
      $data_attributes = array_filter($parsed_url['query'], fn(string $key): bool => str_starts_with($key, 'data'), ARRAY_FILTER_USE_KEY);
      $attributes['style'] = (string) (new Attribute(array_filter($parsed_url['query'], fn(string $value, string $key): bool => str_starts_with($key, '--swiffy-slider'), ARRAY_FILTER_USE_BOTH)));
      $attributes = array_merge(array_filter($attributes->toArray()), $data_attributes);
      $attributes = new Attribute($attributes);
    }
    return $attributes;
  }

}
